use strict;
use warnings;
use Module::Build;

my $builder = Module::Build->new(
    module_name         => 'Statistics::ANOVA',
    license             => 'perl',
    dist_author         => 'Roderick Garton <rgarton@cpan.org>',
    dist_version_from   => 'lib/Statistics/ANOVA.pm',
    build_requires => {
      'Algorithm::Combinatorics' => '0.23',
      'List::AllUtils' => '0.09', # "AllUtils" in case later versions require more than in "Utils" itself
      'Math::Cephes' => '0.43',
      'Scalar::Util' => '1.18',
      'Statistics::Lite' => '3.2',
      'Statistics::Data' => '0.08',
      'Statistics::DependantTTest' => '0.03',
      'Statistics::Descriptive' => '2.6',         
      'Statistics::TTest' => '1.1',
      'Test::More' => '0.0',
    },
    configure_requires => {
        'Module::Build' => '0.42',
    },
    add_to_cleanup      => [ 'Statistics-ANOVA-0.091-*' ],
);

$builder->create_build_script();
